﻿using System;
using System.Runtime.InteropServices;
using System.Text;
using Interop.Plantronics;

namespace Plantronics_SDK_Headset_Proximity_Sample
{
    class Program
    {
        static ICOMDeviceEventsExt_Event _events;
        static ICOMHostCommandExt _command;
        static ICOMDeviceListenerEvents_Event _devListener;

        private static COMDevice _device;

        static void Main()
        {
            //Get a valid _device
            COMSessionManager mgr = new COMSessionManager();
            COMSession sess;
            mgr.Register("My Session", out sess);
            _device = sess.GetActiveDevice();

            //Cast HostCommand and DeviceListener and init _command, _events and _devListener.
            _command = _device.HostCommand as ICOMHostCommandExt;
            _events = _device as ICOMDeviceEventsExt_Event;
            _devListener = _device.DeviceListener;

            Menu();
            int choice = 0;
            while (choice != 6)
            {
                choice = Convert.ToInt32(Console.ReadLine());

                switch (choice)
                {
                    case 1:
                        RequestProximity();
                        break;
                    case 2:
                        if (_events != null)
                        {
                            _events.onHeadsetStateChanged += events_HeadsetStateChanged;
                            Console.WriteLine("Registered (ICOMDeviceEventsExt).");
                        }
                        break;
                    case 3:
                        if (_events != null)
                        {
                            _events.onHeadsetStateChanged -= events_HeadsetStateChanged;
                            Console.WriteLine("Unregistered (ICOMDeviceEventsExt).");
                        }
                        break;
                    case 4:
                        if (_devListener != null)
                        {
                            _devListener.onHeadsetStateChanged += devListener_HeadsetStateChanged;
                            Console.WriteLine("Registered (ICOMDeviceListenerEvents).");
                        }
                        break;
                    case 5:
                        if (_devListener != null)
                        {
                            _devListener.onHeadsetStateChanged -= devListener_HeadsetStateChanged;
                            Console.WriteLine("Unregistered (ICOMDeviceListenerEvents).");
                        }
                        break;
                    default:
                        Console.WriteLine("Please enter a value between 1 and 6. \r\n");
                        Menu();
                        break;
                }
            }
        }

        private static void devListener_HeadsetStateChanged(COMDeviceListenerEventArgs args)
        {
            StringBuilder strB = new StringBuilder();
            strB.Append("Device Event: ");
            strB.Append(String.Format("Headset state changed. ({0})", args.HeadsetStateChange));
            strB.AppendLine();

            Console.WriteLine(strB.ToString());
        }

        private static void events_HeadsetStateChanged(COMHeadsetStateEventArgs args)
        {
            StringBuilder strB = new StringBuilder();
            strB.Append("Device Event: ");
            if (args.HeadsetState == DeviceHeadsetState.HeadsetState_Proximity)
                strB.Append(String.Format("Headset Proximity received. (Proximity = {0})", args.Proximity));
            else
                strB.Append(String.Format("Headset state changed. ({0})", args.HeadsetState));
            strB.AppendLine();

            Console.WriteLine(strB.ToString());
        }

        private static void Menu()
        {
            Console.WriteLine("Plantronics SDK Headset Proximity Sample\r\n" +
                              "\r\n" +
                              "1. Request Headset Proximity. \r\n" +
                              "2. Register Headset Proximity Events (ICOMDeviceEventExt). \r\n" +
                              "3. Unregister Headset Proximity Events (ICOMDeviceEventExt). \r\n" +
                              "4. Register Headset Proximity Events (ICOMDeviceListenerEvents. \r\n" +
                              "5. Unregister Headset Proximity Events (ICOMDeviceListenetEvents. \r\n" +
                              "6. Exit application.");
        }

        private static void RequestProximity()
        {
            try
            {
                if (_command != null)
                {
                    _command.RequestProximity();
                    Console.WriteLine("Proximity requested.");
                }
            }
            catch (COMException ex)
            {
                Console.WriteLine(ex.Message + "\r\nPlease use a _device that supports proximity.");
            }
        }
    }
}
